
import p2.Reporter
from p2.driver.k8s import k8s


class Reporter(p2.Reporter.Reporter):


    def prepare_report_rows(self, context):
        """
        Workflow:
        * get current context
        * initialize access_review_status
        * get all namespaces matching namespaces section
        * for each namespace:
            * for each rule:
                * for each rule.resources:
                    * for each rule.verbs:
                        * get access review
                        * add access review status to access_review_status
        """
        self.logger.info(f"Getting reports for context {context}")
        k8_api = k8s.Api()
        context_config = self.config.get(context)
        if context_config is None:
            self.logger.error(f"No configuration found for context {context}")
        access_review_records = list()
        self.logger.info("Getting all namespaces matching pattern {}...".format(
            context_config["namespaces"]["pattern"]))
        namespaces = k8_api.namespaces(
            pattern=context_config["namespaces"].get('pattern', ".*"),
            skip=context_config["namespaces"].get('skip', [])
            )
        self.logger.info("Found {} total namespaces...".format(len(namespaces)))
        for namespace in namespaces:
            self.logger.info(f"Getting access status on namespace: {namespace}...")
            for rule in context_config["rules"]:
                for group in rule['apiGroups']:
                    for resource in rule["resources"]:
                        for verb in rule["verbs"]:
                            record={
                                "area": namespace["area"],
                                "area_owner": namespace.get('area_owner', 'not set'),
                                "team": namespace.get('team', 'not set'),
                                "area_owner_email": namespace["area_owner_email"],
                                "namespace": namespace["name"],
                                "context": context,
                                "resource": resource,
                                "verb": verb,
                                "allowed": k8_api.access_review(
                                    namespace=namespace["name"],
                                    group=group,
                                    resource=resource,
                                    verb=verb
                                )
                            }
                            self.logger.info("Namespace: {}, group: {}, resource: {}, verb {}: {}".format(
                                namespace['name'], group, resource, verb, record["allowed"]))
                            access_review_records.append(record)
        return access_review_records
